import { ControlElement, Layout, JsonSchema, UISchemaElement, OwnPropsOfMasterListItem, JsonFormsRendererRegistryEntry, JsonFormsCellRendererRegistryEntry } from '@jsonforms/core';
/**
 * Constructs a props declaration for Vue components which can be used
 * for registered renderers and cells. These are typically used in combination
 * with one of the provided bindings, e.g. 'useJsonFormsControl'.
 *
 * Use the generic type parameter when using a specialized binding, e.g.
 * `rendererProps<Layout>()` in combination with `useJsonFormsLayout` or
 * `rendererProps<ControlElement>()` in combination with `useJsonFormsControl`.
 */
export declare const rendererProps: <U = UISchemaElement>() => {
    schema: {
        required: true;
        type: import("vue").PropType<JsonSchema>;
    };
    uischema: {
        required: true;
        type: import("vue").PropType<U>;
    };
    path: {
        required: true;
        type: StringConstructor;
    };
    enabled: {
        required: false;
        type: BooleanConstructor;
        default: undefined;
    };
    renderers: {
        required: boolean;
        type: import("vue").PropType<JsonFormsRendererRegistryEntry[]>;
        default: undefined;
    };
    cells: {
        required: boolean;
        type: import("vue").PropType<JsonFormsCellRendererRegistryEntry[]>;
        default: undefined;
    };
};
/**
 * Constructs a props declaration for Vue components which shall be used as
 * master list items.
 */
export declare const masterListItemProps: () => {
    index: {
        required: true;
        type: NumberConstructor;
    };
    selected: {
        required: true;
        type: BooleanConstructor;
    };
    path: {
        required: true;
        type: StringConstructor;
    };
    schema: {
        required: true;
        type: import("vue").PropType<JsonSchema>;
    };
    handleSelect: {
        required: false;
        type: import("vue").PropType<(index: number) => void>;
        default: undefined;
    };
    removeItem: {
        required: false;
        type: import("vue").PropType<(path: string, value: number) => void>;
        default: undefined;
    };
};
export interface RendererProps<U = UISchemaElement> {
    schema: JsonSchema;
    uischema: U;
    path: string;
    enabled?: boolean;
    renderers?: JsonFormsRendererRegistryEntry[];
    cells?: JsonFormsCellRendererRegistryEntry[];
}
export interface ControlProps extends RendererProps {
    uischema: ControlElement;
}
/**
 * Provides generic bindings for 'Control' elements.
 * Should be used when no specialized bindings are appropriate.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsControl: (props: ControlProps) => {
    control: {
        cells: {
            tester: import("@jsonforms/core").RankedTester;
            cell: any;
        }[];
        label: string | import("@jsonforms/core").Labels;
        description: string;
        required: boolean;
        uischema: ControlElement;
        errors: string;
        data: any;
        rootSchema: JsonSchema;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
    };
} & import("@jsonforms/core").DispatchPropsOfControl;
/**
 * Provides bindings for 'Control' elements which can provide a 'detail',
 * for example array and object renderers.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsControlWithDetail: (props: ControlProps) => {
    control: {
        uischemas: import("@jsonforms/core").JsonFormsUISchemaRegistryEntry[];
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
        label: string | import("@jsonforms/core").Labels;
        description: string;
        required: boolean;
        uischema: ControlElement;
        errors: string;
        data: any;
        rootSchema: JsonSchema;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
    };
} & import("@jsonforms/core").DispatchPropsOfControl;
/**
 * Provides bindings for 'Control' elements which resolve to 'enum' schema elements.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsEnumControl: (props: ControlProps) => {
    control: {
        cells: {
            tester: import("@jsonforms/core").RankedTester;
            cell: any;
        }[];
        label: string | import("@jsonforms/core").Labels;
        description: string;
        required: boolean;
        uischema: ControlElement;
        errors: string;
        data: any;
        rootSchema: JsonSchema;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
        options: import("@jsonforms/core").EnumOption[];
    };
} & import("@jsonforms/core").DispatchPropsOfControl;
/**
 * Provides bindings for 'Control' elements which resolve to manually constructed
 * 'oneOf' enums. These are used to enhance enums with label support.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsOneOfEnumControl: (props: ControlProps) => {
    control: {
        cells: {
            tester: import("@jsonforms/core").RankedTester;
            cell: any;
        }[];
        label: string | import("@jsonforms/core").Labels;
        description: string;
        required: boolean;
        uischema: ControlElement;
        errors: string;
        data: any;
        rootSchema: JsonSchema;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
        options: import("@jsonforms/core").EnumOption[];
    };
} & import("@jsonforms/core").DispatchPropsOfControl;
/**
 * Provides bindings for 'Control' elements which resolve to 'array' schema elements.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsArrayControl: (props: ControlProps) => {
    control: {
        childErrors: import("ajv").ErrorObject[];
        uischemas: import("@jsonforms/core").JsonFormsUISchemaRegistryEntry[];
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
        label: string | import("@jsonforms/core").Labels;
        description: string;
        required: boolean;
        uischema: ControlElement;
        errors: string;
        data: any;
        rootSchema: JsonSchema;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
    };
} & import("@jsonforms/core").DispatchPropsOfArrayControl;
/**
 * Provides bindings for 'Control' elements which resolve to 'allOf' schema elements.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsAllOfControl: (props: ControlProps) => {
    control: {
        rootSchema: JsonSchema;
        path: string;
        id: string;
        indexOfFittingSchema: number;
        uischemas: import("@jsonforms/core").JsonFormsUISchemaRegistryEntry[];
        data: any;
        uischema: ControlElement;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
    };
} & import("@jsonforms/core").DispatchPropsOfControl;
/**
 * Provides bindings for 'Control' elements which resolve to 'anyOf' schema elements.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsAnyOfControl: (props: ControlProps) => {
    control: {
        rootSchema: JsonSchema;
        path: string;
        id: string;
        indexOfFittingSchema: number;
        uischemas: import("@jsonforms/core").JsonFormsUISchemaRegistryEntry[];
        data: any;
        uischema: ControlElement;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
    };
} & import("@jsonforms/core").DispatchPropsOfControl;
/**
 * Provides bindings for 'Control' elements which resolve to 'oneOf' schema elements.
 *
 * Access bindings via the provided reactive `control` object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsOneOfControl: (props: ControlProps) => {
    control: {
        rootSchema: JsonSchema;
        path: string;
        id: string;
        indexOfFittingSchema: number;
        uischemas: import("@jsonforms/core").JsonFormsUISchemaRegistryEntry[];
        data: any;
        uischema: ControlElement;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
    };
} & import("@jsonforms/core").DispatchPropsOfControl;
export interface LayoutProps extends RendererProps {
    uischema: Layout;
}
/**
 * Provides bindings for 'Layout' elements, e.g. VerticalLayout, HorizontalLayout, Group.
 *
 * Access bindings via the provided reactive 'layout' object.
 */
export declare const useJsonFormsLayout: (props: LayoutProps) => {
    layout: {
        direction: "row" | "column";
        config: any;
        uischema: UISchemaElement;
        schema: JsonSchema;
        data: any;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
    };
};
/**
 * Provides bindings for 'Control' elements which resolve to 'array' elements which
 * shall be rendered as a layout instead of a control.
 *
 * Access bindings via the provided reactive 'layout' object.
 */
export declare const useJsonFormsArrayLayout: (props: ControlProps) => {
    layout: {
        data: number;
        minItems: number;
        uischemas: import("@jsonforms/core").JsonFormsUISchemaRegistryEntry[];
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
        label: string | import("@jsonforms/core").Labels;
        description: string;
        required: boolean;
        uischema: ControlElement;
        errors: string;
        rootSchema: JsonSchema;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
    };
};
/**
 * Provides bindings for list elements of a master-list-detail control setup.
 * The element using this binding is not supposed to be registered as an own renderer
 * but used in a more specialized control.
 *
 * Access bindings via the provided reactive 'item' object.
 */
export declare const useJsonFormsMasterListItem: (props: OwnPropsOfMasterListItem) => {
    item: {
        schema: JsonSchema;
        path: string;
        index: number;
        selected: boolean;
    };
};
/**
 * Provides specialized bindings which can be used for any renderer.
 * Useful for meta elements like dispatchers.
 *
 * Access bindings via the provided reactive 'renderer' object.
 * Offers a 'refResolver' method to trigger json-schema-ref-resolver.
 */
export declare const useJsonFormsRenderer: (props: RendererProps<UISchemaElement>) => {
    renderer: {
        cells: JsonFormsCellRendererRegistryEntry[];
        uischema: UISchemaElement;
        rootSchema: JsonSchema;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
        uischemas: import("@jsonforms/core").JsonFormsUISchemaRegistryEntry[];
    };
    refResolver: (schema: any) => any;
};
/**
 * Provides bindings for cell elements. Cells are meant to show simple inputs,
 * for example without error validation, within a larger structure like tables.
 *
 * Access bindings via the provided reactive 'cell' object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsCell: (props: ControlProps) => {
    handleChange(path: string, value: any): void;
    cell: {
        isValid: boolean;
        rootSchema: JsonSchema;
        uischema: ControlElement;
        errors: string;
        data: any;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
    };
};
/**
 * Provides bindings for enum cell elements. Cells are meant to show simple inputs,
 * for example without error validation, within a larger structure like tables.
 *
 * Access bindings via the provided reactive 'cell' object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsEnumCell: (props: ControlProps) => {
    handleChange(path: string, value: any): void;
    cell: {
        isValid: boolean;
        rootSchema: JsonSchema;
        uischema: ControlElement;
        errors: string;
        data: any;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
        cells: JsonFormsCellRendererRegistryEntry[];
        options: import("@jsonforms/core").EnumOption[];
    };
};
/**
 * Provides bindings for a cell dispatcher. Cells are meant to show simple inputs,
 * for example without error validation, within a larger structure like tables.
 *
 * Access bindings via the provided reactive 'cell' object.
 * Dispatch changes via the provided `handleChange` method.
 */
export declare const useJsonFormsDispatchCell: (props: ControlProps) => {
    handleChange(path: string, value: any): void;
    cell: {
        cells: JsonFormsCellRendererRegistryEntry[];
        isValid: boolean;
        rootSchema: JsonSchema;
        uischema: ControlElement;
        errors: string;
        data: any;
        id: string;
        config: any;
        schema: JsonSchema;
        enabled: boolean;
        visible: boolean;
        path: string;
        renderers: JsonFormsRendererRegistryEntry[];
    };
};
